@@ -8,7 +8,9 @@ import android.net.Uri;
 import android.os.Build;
 import android.os.Bundle;
 import android.os.Parcelable;
+import android.text.Editable;
 import android.text.TextUtils;
+import android.text.TextWatcher;
 import android.view.MenuItem;
 import android.view.View;
 import android.view.ViewGroup;
@@ -37,11 +39,14 @@ import com.seafile.seadroid2.databinding.ToolbarActionbarBinding;
 import com.seafile.seadroid2.framework.model.docs_comment.DocsCommentModel;
 import com.seafile.seadroid2.framework.model.docs_comment.DocsCommentsWrapperModel;
 import com.seafile.seadroid2.framework.model.sdoc.SDocPageOptionsModel;
+import com.seafile.seadroid2.framework.model.user.UserModel;
 import com.seafile.seadroid2.framework.util.SLogs;
 import com.seafile.seadroid2.framework.util.Toasts;
 import com.seafile.seadroid2.ui.base.BaseMediaSelectorActivity;
+import com.seafile.seadroid2.ui.dialog_fragment.related_users.RelatedUserBottomSheetDialogFragment;
 import com.seafile.seadroid2.view.rich_edittext.RichEditText;
 
+import java.util.HashMap;
 import java.util.List;
 
 import io.reactivex.functions.Consumer;
@@ -54,16 +59,19 @@ public class DocsCommentsActivity extends BaseMediaSelectorActivity<DocsCommentV
     private DocsCommentUserAdapter userAdapter;
 
     private SDocPageOptionsModel pageOptionsModel;
+    private boolean hasModifyPermission = false;
 
-    public static void start(Context context, SDocPageOptionsModel pageModel) {
+    public static void start(Context context, SDocPageOptionsModel pageModel, boolean hasModifyPermission) {
         Intent starter = new Intent(context, DocsCommentsActivity.class);
         starter.putExtra("pageOption", pageModel);
+        starter.putExtra("hasModifyPermission", hasModifyPermission);
         context.startActivity(starter);
     }
 
     @Override
     protected void onSaveInstanceState(@NonNull Bundle outState) {
         super.onSaveInstanceState(outState);
+        outState.putBoolean("hasModifyPermission", hasModifyPermission);
         outState.putParcelable("pageOption", pageOptionsModel);
         Parcelable listParcelable = linearLayoutManager.onSaveInstanceState();
         outState.putParcelable("listParcelable", listParcelable);
@@ -81,14 +89,8 @@ public class DocsCommentsActivity extends BaseMediaSelectorActivity<DocsCommentV
 
         applyEdgeToEdge(binding.getRoot());
 
-        initView();
         adaptInputMethod();
 
-        initViewModel();
-
-        initAdapter();
-
-
         if (savedInstanceState != null) {
             pageOptionsModel = savedInstanceState.getParcelable("pageOption");
             if (pageOptionsModel == null) {
@@ -100,6 +102,8 @@ public class DocsCommentsActivity extends BaseMediaSelectorActivity<DocsCommentV
             if (listParcelable != null) {
                 linearLayoutManager.onRestoreInstanceState(listParcelable);
             }
+
+            hasModifyPermission = savedInstanceState.getBoolean("hasModifyPermission", false);
         } else {
             if (getIntent() == null || !getIntent().hasExtra("pageOption")) {
                 throw new IllegalArgumentException("pageOption is null");
@@ -110,8 +114,16 @@ public class DocsCommentsActivity extends BaseMediaSelectorActivity<DocsCommentV
                 throw new IllegalArgumentException("pageOption is null");
             }
             bindingOfToolbar.toolbarActionbar.setTitle(pageOptionsModel.docName);
+
+            hasModifyPermission = getIntent().getBooleanExtra("hasModifyPermission", false);
         }
 
+        initViewModel();
+
+        initAdapter();
+
+        initView();
+
         refreshData();
     }
 
@@ -133,23 +145,51 @@ public class DocsCommentsActivity extends BaseMediaSelectorActivity<DocsCommentV
 
         binding.rv.setLayoutManager(linearLayoutManager);
 
-        binding.photoView.setOnClickListener(new View.OnClickListener() {
-            @Override
-            public void onClick(View v) {
-                showPickPhotoSheetDialog(false);
-            }
-        });
+        if (hasModifyPermission) {
+            binding.photoView.setOnClickListener(new View.OnClickListener() {
+                @Override
+                public void onClick(View v) {
+                    showPickPhotoSheetDialog(false);
+                }
+            });
 
 //        //
 //        LinearLayoutManager linearLayoutManager = new LinearLayoutManager(this, LinearLayoutManager.HORIZONTAL, false);
 //        binding.rvUserList.setLayoutManager(linearLayoutManager);
 
-        binding.submit.setOnClickListener(new View.OnClickListener() {
-            @Override
-            public void onClick(View v) {
-                submitData();
-            }
-        });
+            binding.submit.setOnClickListener(new View.OnClickListener() {
+                @Override
+                public void onClick(View v) {
+                    submitData();
+                }
+            });
+
+            binding.richEditText.setTextWatcher(new TextWatcher() {
+                @Override
+                public void beforeTextChanged(CharSequence s, int start, int count, int after) {
+
+                }
+
+                @Override
+                public void onTextChanged(CharSequence s, int start, int before, int count) {
+                    if (count == 1 && before == 0) {
+                        char insertedChar = s.charAt(start);
+                        if (insertedChar == '@') {
+                            SLogs.d("DocsCommentsActivity", "@ character detected. Showing user selector...");
+                            showRelateUserDialog();
+                        }
+                    }
+                }
+
+                @Override
+                public void afterTextChanged(Editable s) {
+
+                }
+            });
+        } else {
+            binding.bottomSheetContainer.setVisibility(View.GONE);
+        }
+
 //
 //        binding.richEditText.setOnRichAtListener(new OnRichAtListener() {
 //            @Override
@@ -233,6 +273,13 @@ public class DocsCommentsActivity extends BaseMediaSelectorActivity<DocsCommentV
             }
         });
 
+        getViewModel().getRelatedUsersLiveData().observe(this, new Observer<List<UserModel>>() {
+            @Override
+            public void onChanged(List<UserModel> userModels) {
+
+            }
+        });
+
         getViewModel().getSeafExceptionLiveData().observe(this, new Observer<SeafException>() {
             @Override
             public void onChanged(SeafException e) {
@@ -347,8 +394,25 @@ public class DocsCommentsActivity extends BaseMediaSelectorActivity<DocsCommentV
         builder.show();
     }
 
+    private final HashMap<String, UserModel> toNotifyUserMap = new HashMap<>();
+
+    private void showRelateUserDialog() {
+        RelatedUserBottomSheetDialogFragment fragment = new RelatedUserBottomSheetDialogFragment();
+        fragment.setOnItemClickListener(new BaseQuickAdapter.OnItemClickListener<UserModel>() {
+            @Override
+            public void onClick(@NonNull BaseQuickAdapter<UserModel, ?> baseQuickAdapter, @NonNull View view, int i) {
+                UserModel userModel = baseQuickAdapter.getItems().get(i);
+                binding.richEditText.addText(userModel.getName() + " ");
+
+                toNotifyUserMap.put(userModel.getEmail(), userModel);
+            }
+        });
+        fragment.show(getSupportFragmentManager(), RelatedUserBottomSheetDialogFragment.class.getName());
+    }
+
     private void refreshData() {
         getViewModel().loadDocComments(pageOptionsModel);
+        getViewModel().getRelatedUsers(pageOptionsModel.repoID);
     }
 
     public void onMediaPicked(Uri uri) {
@@ -406,6 +470,6 @@ public class DocsCommentsActivity extends BaseMediaSelectorActivity<DocsCommentV
         }
 
         // 0 is root comment
-        getViewModel().postComment(pageOptionsModel, sb.toString(), "0");
+        getViewModel().postComment(pageOptionsModel, sb.toString());
     }
 }
