@@ -14,11 +14,12 @@ import android.os.Bundle;
 import android.os.IBinder;
 import android.text.TextUtils;
 import android.view.MenuItem;
+import android.widget.LinearLayout;
 
 import androidx.activity.OnBackPressedCallback;
 import androidx.annotation.NonNull;
 import androidx.appcompat.app.AppCompatDelegate;
-import androidx.constraintlayout.widget.ConstraintLayout;
+import androidx.core.content.ContextCompat;
 import androidx.core.graphics.Insets;
 import androidx.core.view.ViewCompat;
 import androidx.core.view.WindowInsetsCompat;
@@ -31,7 +32,6 @@ import com.blankj.utilcode.util.ActivityUtils;
 import com.blankj.utilcode.util.NetworkUtils;
 import com.google.android.material.navigation.NavigationBarView;
 import com.seafile.seadroid2.R;
-import com.seafile.seadroid2.SeadroidApplication;
 import com.seafile.seadroid2.account.Account;
 import com.seafile.seadroid2.account.SupportAccountManager;
 import com.seafile.seadroid2.bus.BusAction;
@@ -40,21 +40,18 @@ import com.seafile.seadroid2.context.GlobalNavContext;
 import com.seafile.seadroid2.context.NavContext;
 import com.seafile.seadroid2.databinding.ActivityMainBinding;
 import com.seafile.seadroid2.enums.NightMode;
+import com.seafile.seadroid2.framework.file_monitor.FileDaemonService;
 import com.seafile.seadroid2.framework.file_monitor.FileSyncService;
 import com.seafile.seadroid2.framework.model.ServerInfo;
-import com.seafile.seadroid2.framework.service.BackupThreadExecutor;
 import com.seafile.seadroid2.framework.util.PermissionUtil;
 import com.seafile.seadroid2.framework.util.SLogs;
-import com.seafile.seadroid2.framework.worker.BackgroundJobManagerImpl;
 import com.seafile.seadroid2.preferences.Settings;
 import com.seafile.seadroid2.ui.account.AccountsActivity;
 import com.seafile.seadroid2.ui.activities.AllActivitiesFragment;
 import com.seafile.seadroid2.ui.adapter.ViewPager2Adapter;
 import com.seafile.seadroid2.ui.base.BaseActivity;
-import com.seafile.seadroid2.ui.camera_upload.CameraUploadManager;
 import com.seafile.seadroid2.ui.repo.RepoQuickFragment;
 
-import java.io.File;
 import java.util.List;
 import java.util.Optional;
 
@@ -131,7 +128,7 @@ public class MainActivity extends BaseActivity {
             Insets systemBars = insets.getInsets(WindowInsetsCompat.Type.systemBars());
             v.setPadding(systemBars.left, 0, systemBars.right, 0);
 
-            ConstraintLayout.LayoutParams params = (ConstraintLayout.LayoutParams) binding.statusBarGuideline.getLayoutParams();
+            LinearLayout.LayoutParams params = (LinearLayout.LayoutParams) binding.statusBarGuideline.getLayoutParams();
             params.height = systemBars.top;
             binding.statusBarGuideline.setLayoutParams(params);
             return insets;
@@ -201,7 +198,7 @@ public class MainActivity extends BaseActivity {
     }
 
     private void requestNotificationPermission() {
-        if (PermissionUtil.hasNotificationPermission(this)) {
+        if (PermissionUtil.hasNotGrantNotificationPermission(this)) {
             PermissionUtil.requestNotificationPermission(this);
         }
     }
@@ -262,18 +259,13 @@ public class MainActivity extends BaseActivity {
         BusHelper.getCommonObserver().removeObserver(commonBusObserver);
         BusHelper.getNavContextObserver().removeObserver(navContextObserver);
 
-        BackgroundJobManagerImpl.getInstance().stopAlbumBackupPeriodicScan(SeadroidApplication.getAppContext());
-        BackgroundJobManagerImpl.getInstance().stopFolderBackupPeriodicScan(SeadroidApplication.getAppContext());
-        //
+//        BackgroundJobManagerImpl.getInstance().stopAlbumBackupPeriodicScan(SeadroidApplication.getAppContext());
+//        BackgroundJobManagerImpl.getInstance().stopFolderBackupPeriodicScan(SeadroidApplication.getAppContext());
 //        BackgroundJobManagerImpl.getInstance().cancelAllJobs();
 
 
-        if (isBound) {
-            unbindService(syncConnection);
-            isBound = false;
-            syncService = null;
-        }
-
+        unbindService();
+        //
         super.onDestroy();
     }
 
@@ -445,11 +437,25 @@ public class MainActivity extends BaseActivity {
 
     // service
     private void bindService() {
+        Intent syncIntent = new Intent(this, FileSyncService.class);
         if (!isBound) {
-            Intent syncIntent = new Intent(this, FileSyncService.class);
             bindService(syncIntent, syncConnection, Context.BIND_AUTO_CREATE);
             isBound = true;
         }
+
+        boolean isTurnOn = Settings.BACKUP_SETTINGS_BACKGROUND_SWITCH.queryValue();
+        if (isTurnOn) {
+            Intent daemonIntent = new Intent(this, FileDaemonService.class);
+            ContextCompat.startForegroundService(this, daemonIntent);
+        }
+    }
+
+    private void unbindService() {
+        if (isBound) {
+            unbindService(syncConnection);
+            isBound = false;
+            syncService = null;
+        }
     }
 
     private boolean isBound = false;
@@ -458,7 +464,6 @@ public class MainActivity extends BaseActivity {
         public void onServiceConnected(ComponentName name, IBinder service) {
             FileSyncService.FileSyncBinder binder = (FileSyncService.FileSyncBinder) service;
             syncService = binder.getService();
-            syncService.setFileChangeListener(onLocalFileChangeListener);
             SLogs.d(TAG, "bond FileSyncService");
         }
 
@@ -470,26 +475,6 @@ public class MainActivity extends BaseActivity {
         }
     };
 
-    private final FileSyncService.FileChangeListener onLocalFileChangeListener = new FileSyncService.FileChangeListener() {
-        @Override
-        public void onLocalFileChanged(File fileToScan) {
-            SLogs.d(TAG, "onLocalFileChanged", fileToScan.getAbsolutePath());
-            BackupThreadExecutor.getInstance().runLocalFileUpdateTask();
-        }
-
-        @Override
-        public void onBackupFileChanged(File fileToScan) {
-            SLogs.d(TAG, "onBackupFileChanged", fileToScan.getAbsolutePath());
-            BackupThreadExecutor.getInstance().runFolderBackupFuture(true);
-        }
-
-        @Override
-        public void onMediaContentObserver(boolean isFullScan) {
-            SLogs.d(TAG, "onMediaContentObserver", "isFullScan: " + isFullScan);
-            CameraUploadManager.getInstance().performSync(isFullScan);
-        }
-    };
-
     // check server info
     private void requestServerInfo(boolean loadFromNet) {
         Optional<ServerInfo> optional = checkServerInfo();
@@ -680,6 +665,14 @@ public class MainActivity extends BaseActivity {
                 if (syncService != null) {
                     syncService.restartFolderMonitor();
                 }
+            } else if (TextUtils.equals(action, BusAction.START_FOREGROUND_FILE_MONITOR)) {
+
+                Intent intent = new Intent(MainActivity.this, FileDaemonService.class);
+                ContextCompat.startForegroundService(MainActivity.this, intent);
+
+            } else if (TextUtils.equals(action, BusAction.STOP_FOREGROUND_FILE_MONITOR)) {
+                Intent intent = new Intent(MainActivity.this, FileDaemonService.class);
+                stopService(intent);
             }
         }
     };
